@@ -1,9 +1,5 @@
-#include <sys/types.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <limits.h>
 #include <libnjb.h>
+#include <getopt.h>
 
 void usage (void);
 
