@@ -1,9 +1,6 @@
-#include <stdio.h>
-#include <sys/types.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <limits.h>
+#include <string.h>
 #include <libnjb.h>
+#include <getopt.h>
 
 extern int njb_error;
 
