@@ -1,7 +1,6 @@
-#include <sys/types.h>
-#include <unistd.h>
+#include <string.h>
 #include <libnjb.h>
-#include <stdlib.h>
+#include <getopt.h>
 
 int main (int argc, char **argv)
 {
