@@ -1,7 +1,5 @@
-#include <sys/types.h>
-#include <unistd.h>
 #include <libnjb.h>
-#include <stdlib.h>
+#include <getopt.h>
 
 int main (int argc, char **argv)
 {
@@ -43,10 +41,17 @@ int main (int argc, char **argv)
        		return 1;
 	}
 
+#ifdef __WIN32__
+       	printf("Total bytes on jukebox: %I64u (%I64u MB)\n",
+	       total, total/(1024*1024));
+       	printf("Free bytes on jukebox: %I64u (%I64u MB)\n",
+	       free, free/(1024*1024));
+#else
        	printf("Total bytes on jukebox: %llu (%llu MB)\n",
 	       total, total/(1024*1024));
        	printf("Free bytes on jukebox: %llu (%llu MB)\n",
 	       free, free/(1024*1024));
+#endif
 
        	NJB_Close(njb);
 
