@@ -1,9 +1,5 @@
-#include <stdio.h>
-#include <sys/types.h>
-#include <stdlib.h>
-#include <limits.h>
 #include <libnjb.h>
-#include <unistd.h>
+#include <getopt.h>
 
 extern int njb_error;
 
@@ -48,11 +44,14 @@ int main (int argc, char **argv)
 	}
 
 	NJB_Reset_Get_Playlist(njb);
+	n = 0;
 	while ( (playlist= NJB_Get_Playlist(njb)) ) {
 		playlist_dump(playlist, stdout);
 		printf("----------------------------------\n");
+		n ++;
 	}
 	if ( njb_error != EO_EOM ) njb_error_dump(stderr);
+	printf("Total: %d playlists\n", n);
 
 	NJB_Release(njb);
 
