@@ -1,12 +1,8 @@
 #ifndef __NJB__BASE__H
 #define __NJB__BASE__H
 
-#include "../config.h"
-#include <stdio.h>
 #include "libnjb.h"
 
-#define TMPDIR "/tmp"
-
 int njb_discover (njb_t *njbs, int limit, int *errorflag);
 void njb_device_dump (njb_t *njb, FILE *fp);
 int njb_open (njb_t *njb);
