@@ -1,14 +1,10 @@
-#include "../config.h"
-#include <sys/types.h>
-#include <sys/timeb.h>
-#include <sys/stat.h>
-#include <ctype.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <sys/stat.h> /* stat() */
+#include <ctype.h> /* isspace() */
 #include <fcntl.h>
-#include <libgen.h>
-#include <unistd.h>
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h> /* basename() */
+#endif
+#include <string.h> /* Various memory and string functions */
 #include "libnjb.h"
 #include "procedure.h"
 #include "protocol.h"
@@ -20,6 +16,7 @@
 #include "ioutil.h"
 #include "unicode.h"
 #include "byteorder.h"
+#include "eax.h"
 
 static int _lib_ctr_update (njb_t *njb);
 void _skip_whitespaces (FILE *);
@@ -32,14 +29,28 @@ extern int njb_error;
 extern int njb_unicode_flag;
 extern int __sub_depth;
 
+/* Function that compensate for missing libgen.h on Windows */
+#ifndef HAVE_LIBGEN_H
+static char *basename(char *in) {
+  char *p;
+  if (in == NULL)
+    return NULL;
+  p = in + strlen(in) - 1;
+  while (*p != '\\' && *p != '/' && *p != ':')
+    { p--; }
+  return ++p;
+}
+#endif
+
 static int _lib_ctr_update (njb_t *njb)
 {
 	__dsub= "_lib_ctr_update";
 	njblibctr_t lcount;
+	njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
 	__enter;
 
-	if ( njb->session_updated ) return 0;
+	if ( state->session_updated ) return 0;
 
 	if ( njb_get_library_counter(njb, &lcount) == -1 ) {
 		__leave;
@@ -64,8 +75,8 @@ static int _lib_ctr_update (njb_t *njb)
 		return -1;
 	}
 
-	njb->session_updated= 1;
-	njb->libcount++;
+	state->session_updated = 1;
+	state->libcount ++;
 	__leave;
 	return 0;
 }
@@ -101,6 +112,13 @@ int NJB_Open (njb_t *njb)
 	   * Also see NJB_Close() (similar code). LW.
 	   */
 
+	  if (njb->device_type == NJB_DEVICE_NJB1) {
+	    if ( njb_init_state(njb) == -1) {
+	      __leave;
+	      return -1;
+	    }
+	  }
+
 	  if (PROTOCOL3_DEVICE(njb->device_type)) {
 	    if ( njb3_init_state(njb) == -1) {
 	      __leave;
@@ -164,6 +182,7 @@ int NJB_Capture (njb_t *njb)
 	 */
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 		njblibctr_t lcount;
+		njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
 		njb_error_clear();
 
@@ -177,7 +196,7 @@ int NJB_Capture (njb_t *njb)
 			return -1;
 		}
 
-		if ( njb->libcount != lcount.count ) {
+		if ( state->libcount != lcount.count ) {
 			njb_capture(njb, NJB_RELEASE);
 			NJB_ERROR(EO_BADCOUNT);
 			__leave;
@@ -211,7 +230,6 @@ int NJB_Release (njb_t *njb)
 int NJB_Handshake (njb_t *njb)
 {
 	__dsub= "NJB_Handshake";
-	njblibctr_t lcount, lcount_new, lcount_old;
 	njbid_t njbid;
 	char *cp;
 	int i;
@@ -264,6 +282,9 @@ int NJB_Handshake (njb_t *njb)
 
 	/* No library counters on NJB3 */
 	if (njb->device_type == NJB_DEVICE_NJB1) {
+	        njblibctr_t lcount, lcount_new, lcount_old;
+	        njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
 		if ( njb_get_library_counter(njb, &lcount) == -1 ) {
 			__leave;
 			return -1;
@@ -317,7 +338,7 @@ int NJB_Handshake (njb_t *njb)
 			return -1;
 		}
 
-		njb->libcount= lcount_old.count;
+		state->libcount = lcount_old.count;
 	}
 
 	__leave;
@@ -327,10 +348,10 @@ int NJB_Handshake (njb_t *njb)
 void NJB_Get_Extended_Tags (njb_t *njb, int extended)
 {
   __dsub= "NJB_Get_Extended_Tags";
-
+  njb3_state_t *state = (njb3_state_t *) njb->protocol_state;
   __enter;
 
-  njb->get_extended_tag_info = extended;
+  state->get_extended_tag_info = extended;
 
   __leave;
 }
@@ -343,7 +364,17 @@ void NJB_Reset_Get_Track_Tag (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_track_tag = 1;
+	if (njb->device_type == NJB_DEVICE_NJB1) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_track_tag = 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_track_tag(njb) == -1) {
+	    /* FIXME: do something with this error some day. */
+	  }
+	}
 
 	__leave;
 }
@@ -360,10 +391,11 @@ songid_t *NJB_Get_Track_Tag (njb_t *njb)
 
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 		njbttaghdr_t tagh;
+		njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
-		if ( njb->reset_get_track_tag ) {
+		if ( state->reset_get_track_tag ) {
 			status= njb_get_first_track_tag_header(njb, &tagh);
-			njb->reset_get_track_tag= 0;
+			state->reset_get_track_tag = 0;
 		} else {
 			status= njb_get_next_track_tag_header(njb, &tagh);
 		}
@@ -371,12 +403,7 @@ songid_t *NJB_Get_Track_Tag (njb_t *njb)
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  if ( njb->reset_get_track_tag ) {
-	    ret=njb3_get_first_track_tag(njb, njb->get_extended_tag_info);
-	    njb->reset_get_track_tag= 0;
-	  } else {
-	    ret=njb3_get_next_track_tag(njb);
-	  }
+	  ret = njb3_get_next_track_tag(njb);
 	}
 
 	__leave;
@@ -391,7 +418,17 @@ void NJB_Reset_Get_Playlist (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_playlist= 1;
+	if (njb->device_type == NJB_DEVICE_NJB1) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_playlist= 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_playlist_tag(njb) == -1) {
+	    /* FIXME: do something about this error eventually */
+	  }
+	}
 
 	__leave;
 }
@@ -408,8 +445,9 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 	njb_error_clear();
 
 	if (njb->device_type == NJB_DEVICE_NJB1) {
+	        njb_state_t *state = (njb_state_t *) njb->protocol_state;
 
-		if ( njb->reset_get_playlist ) {
+		if ( state->reset_get_playlist ) {
 			while (retry) {
 				if ( njb_get_first_playlist_header(njb, &plh) == -1 ) {
 					if ( njb_error == EO_AGAIN && retry ) retry--;
@@ -419,7 +457,7 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 					}
 				} else break;
 			}
-			njb->reset_get_playlist= 0;
+			state->reset_get_playlist= 0;
 		} else {
 			while (retry) {
 				if ( njb_get_next_playlist_header(njb, &plh) == -1 ) {
@@ -432,17 +470,11 @@ playlist_t *NJB_Get_Playlist (njb_t *njb)
 			}
 		}
 
-		ret= njb_get_playlist(njb, &plh);
+		ret = njb_get_playlist(njb, &plh);
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  if ( njb->reset_get_playlist ) {
-	    ret = njb3_get_first_playlist_tag (njb);
-	    njb->reset_get_playlist= 0;
-	  }
-	  else {
-	    ret = njb3_get_next_playlist_tag(njb);
-	  }
+	  ret = njb3_get_next_playlist_tag(njb);
 	}
 
 	__leave;
@@ -571,7 +603,17 @@ void NJB_Reset_Get_Datafile_Tag (njb_t *njb)
 
 	njb_error_clear();
 
-	njb->reset_get_datafile_tag= 1;
+	if (njb->device_type == NJB_DEVICE_NJB1 ) {
+	  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+	  state->reset_get_datafile_tag = 1;
+	}
+
+	if (PROTOCOL3_DEVICE(njb->device_type)) {
+	  if (njb3_reset_get_datafile_tag(njb) == -1) {
+	    /* FIXME: handle this error eventually */
+	  }
+	}
 
 	__leave;
 }
@@ -588,73 +630,34 @@ datafile_t *NJB_Get_Datafile_Tag (njb_t *njb)
 	njb_error_clear();
 
 	if (njb->device_type == NJB_DEVICE_NJB1 ) {
-		if ( njb->reset_get_datafile_tag ) {
-			status= njb_get_first_datafile_header(njb, &dfh);
-			njb->reset_get_datafile_tag= 0;
+	        njb_state_t *state = (njb_state_t *) njb->protocol_state;
+
+		if ( state->reset_get_datafile_tag ) {
+			status = njb_get_first_datafile_header(njb, &dfh);
+			state->reset_get_datafile_tag= 0;
 		} else {
-			status= njb_get_next_datafile_header(njb, &dfh);
+			status = njb_get_next_datafile_header(njb, &dfh);
 		}
 
 		ret= ( status == -1 ) ? NULL : njb_get_datafile_tag(njb, &dfh);
 	}
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  if ( njb->reset_get_datafile_tag ) {
-	    ret= njb3_get_first_datafile_tag(njb);
-	    njb->reset_get_datafile_tag= 0;
-	  } else {
-	    ret= njb3_get_next_datafile_tag(njb);
-	  }
+	  ret = njb3_get_next_datafile_tag(njb);
 	}
 
 	__leave;
 	return ret;
 }
 
-int NJB_Set_TmpDir (njb_t *njb, const char *dir)
-{
-	__dsub= "NJB_Set_TmpDir";
-
-	__enter;
-
-	njb_error_clear();
-
-	if ( njb->tmpdir ) free(njb->tmpdir);
-
-	njb->tmpdir= strdup(dir);
-	if ( njb->tmpdir == NULL ) {
-		NJB_ERROR(EO_NOMEM);
-		__leave;
-		return -1;
-	}
-
-	__leave;
-	return 0;
-}
-
-const char *NJB_Get_TmpDir (njb_t *njb)
-{
-	__dsub= "NJB_Get_TmpDir";
-
-	__enter;
-
-	njb_error_clear();
-
-	__leave;
-
-	return (const char *) njb->tmpdir;
-}
-
 int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 	const char *path, XferCallback *callback, void *data)
 {
 	__dsub= "NJB_Get_Track";
-	char *temp = NULL;
 	unsigned char *block = NULL;
 	int fd = 0;
 	u_int32_t bsize, offset;
 	u_int32_t remain = size;
-	u_int32_t callbackcnt, callbackval;
 	int abortxfer = 0;
 	int ret;
 
@@ -662,33 +665,6 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 
 	njb_error_clear();
 
-	if ( path ) {
-		if ( njb->tmpdir == NULL ) {
-			NJB_ERROR(EO_NULLTMP);
-			ret = -1;
-			goto clean_up_and_return;
-		} 
-
-		temp= (char *) malloc
-			(strlen(njb->tmpdir)+strlen(XFER_TEMPLATE)+2);
-
-		if ( temp == NULL ) {
-			NJB_RERROR("malloc", -1);
-			NJB_ERROR(EO_TMPFILE);
-			ret = -1;
-			goto clean_up_and_return;
-		}
-
-		sprintf(temp, "%s/%s", njb->tmpdir, XFER_TEMPLATE);
-
-		if ( mkstemp(temp) == -1 ) {
-			NJB_RERROR("mkstemp", -1);
-			NJB_ERROR(EO_TMPFILE);
-			ret = -1;
-			goto clean_up_and_return;
-		}
-	}
-
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 
 		if ( njb_request_file(njb, fileid) == -1 ) {
@@ -703,49 +679,38 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 		}
 	}
 
-	if ( path && (fd= open(temp, O_CREAT|O_TRUNC|O_WRONLY)) == -1 ) {
+#ifdef __WIN32__
+	if ( path && (fd= open(path, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY, 0664)) == -1 ) {
+#else
+	if ( path && (fd= open(path, O_CREAT|O_TRUNC|O_WRONLY, 0664)) == -1 ) {
+#endif
 		NJB_RERROR("open", -1);
 		NJB_ERROR(EO_TMPFILE);
 		ret = -1;
 		goto clean_up_and_return;
 	}
 
-	/* These two are used to make calls to the callback routine
-	 * no more than CALLBACK_NUM times / song */
-	callbackcnt = size / CALLBACK_NUM;
-	callbackval = 0;
-
 	/* File transfer routine for the NJB1 */
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 	        u_int32_t bread = 0;
 		int wasshort = 0;
 
-		block = (unsigned char *) malloc(MAX_XFER_BLOCK_SIZE + XFER_BLOCK_HEADER_SIZE);
+		block = (unsigned char *) malloc(NJB_XFER_BLOCK_SIZE + NJB_XFER_BLOCK_HEADER_SIZE);
 		if ( block == NULL ) {
 			NJB_ERROR(EO_NOMEM);
 			ret = -1;
 			goto clean_up_and_return;
 		}
-		memset(block, 0, MAX_XFER_BLOCK_SIZE + XFER_BLOCK_HEADER_SIZE);
+		memset(block, 0, NJB_XFER_BLOCK_SIZE + NJB_XFER_BLOCK_HEADER_SIZE);
 		offset = 0;
 		while (remain != 0 && !abortxfer) {
 		        /* Request as much as possible unless the last chunk is reached */
-			bsize= (remain > njb->xfersize) ? njb->xfersize : remain;
+			bsize= (remain > NJB_XFER_BLOCK_SIZE) ? NJB_XFER_BLOCK_SIZE : remain;
 
-			/* Write receieved bytes to file */
-			if (path && bread > XFER_BLOCK_HEADER_SIZE) {
-			  if ( (Write(fd, &block[XFER_BLOCK_HEADER_SIZE], bread-XFER_BLOCK_HEADER_SIZE, 0)) == -1 ) {
-			    NJB_RERROR("Write", -1);
-			    NJB_ERROR(EO_WRFILE);
-			    __leave;
-			    ret = -1;
-			    goto clean_up_and_return;
-			  }
-			}
 			bread = 0;
 
 			bread = njb_receive_file_block(njb, offset, bsize, &block[0]);
-			if (bread < bsize + XFER_BLOCK_HEADER_SIZE) {
+			if (bread < bsize + NJB_XFER_BLOCK_HEADER_SIZE) {
 			  wasshort = 1;
 			} else {
 			  wasshort = 0;
@@ -764,16 +729,27 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 			  fprintf(stdout, " - short read");
 			fprintf(stdout,"\n");
 			*/
-			
-			if ( callback != NULL ) {
-			  const char *last_block= (const char *) &block[XFER_BLOCK_HEADER_SIZE];
-			  if ( callback(offset, size, last_block, bread-XFER_BLOCK_HEADER_SIZE, data) == -1 ) {
-			    abortxfer = 1;
+		
+			/* Write receieved bytes to file */
+			if (path && bread > NJB_XFER_BLOCK_HEADER_SIZE) {
+			  if ( (write(fd, &block[NJB_XFER_BLOCK_HEADER_SIZE], bread-NJB_XFER_BLOCK_HEADER_SIZE)) == -1 ) {
+			    NJB_RERROR("write", -1);
+			    NJB_ERROR(EO_WRFILE);
+			    __leave;
+			    ret = -1;
+			    goto clean_up_and_return;
 			  }
 			}
 
-			remain -= (bread-XFER_BLOCK_HEADER_SIZE);
-			offset += (bread-XFER_BLOCK_HEADER_SIZE);
+			remain -= (bread-NJB_XFER_BLOCK_HEADER_SIZE);
+			offset += (bread-NJB_XFER_BLOCK_HEADER_SIZE);
+
+			if ( callback != NULL ) {
+			  const char *last_block= (const char *) &block[NJB_XFER_BLOCK_HEADER_SIZE];
+			  if ( callback(offset, size, last_block, bread-NJB_XFER_BLOCK_HEADER_SIZE, data) == -1 ) {
+			    abortxfer = 1;
+			  }
+			}
 		}
 
 		/* This is probably not the right way to abort a file transfer */
@@ -790,18 +766,6 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 			ret = -1;
 			goto clean_up_and_return;
 		}
-
-		/* Write last part of the file */
-		if ( path && bread > XFER_BLOCK_HEADER_SIZE) {
-		  if ( (Write(fd, &block[XFER_BLOCK_HEADER_SIZE], bread-XFER_BLOCK_HEADER_SIZE, 0)) == -1 ) {
-		    NJB_RERROR("Write", -1);
-		    NJB_ERROR(EO_WRFILE);
-		    __leave;
-		    ret = -1;
-		    goto clean_up_and_return;
-		  }
-		}
-
 	}
 
 	/* Protocol 3 series file transfer routine */
@@ -840,22 +804,14 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 
 		  chunk_remain = chunk_size;
 
+		  /* Do NOT break on abort here - all blocks must be sent before 
+		   * cancelling the transfer! */
 		  while (chunk_remain != 0) {
 
 		    /*
 		     * Try to get maximum amount, if we run into a short 
 		     * read, that's OK 
 		     */
-
-		    /*
-		    if ((chunk_remain < NJB3_CHUNK_SIZE) 
-			&& (
-			    ((chunk_remain & 0x1FFF) == 0) || ((chunk_remain & 0x1FFF ) == 0x1FFF)
-			    )) {
-		      printf("Dangerous chunk - remaining size: %08X!\n", chunk_remain);
-		    }
-		    */
-
 		    /* printf("Requesting chunk size %08X, remaining %08X...\n", NJB3_SUBCHUNK_SIZE, chunk_remain); */
 		    bread = njb3_get_file_block(njb, block, NJB3_GET_FILE_BLOCK_SIZE);
 		    /* Negative value signals error */
@@ -878,14 +834,14 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 		      }
 		      if (extraneous > 1) {
 			/* This should, however, not happen. */
-			printf("LIBNJB problem: recieved %d extraneous bytes!\n", extraneous);
+			printf("LIBNJB panic: recieved %d extraneous bytes!\n", extraneous);
 		      }
 		      bread = chunk_remain;
 		    }
 		  
 		    /* Write the bytes to the destination file */
-		    if ( (bwritten = Write(fd, block, bread, 0) ) == -1 ) {
-		      NJB_RERROR("Write", -1);
+		    if ( (bwritten = write(fd, block, bread) ) == -1 ) {
+		      NJB_RERROR("write", -1);
 		      NJB_ERROR(EO_WRFILE);
 		      ret = -1;
 		      goto clean_up_and_return;
@@ -897,10 +853,8 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 
 		    chunk_remain -= bread;
 		    offset += bread;
-		    callbackval += bread;
 
-		    if ( (callback != NULL) && (callbackval > callbackcnt) ) {
-		      callbackval = 0;
+		    if ( callback != NULL) {
 		      if ( callback(offset, size, NULL, 0, data) == -1 )
 			abortxfer= 1;
 		    }
@@ -917,13 +871,13 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 		    }
 		    if (extraneous > 1) {
 		      /* This should, however, not happen. */
-		      printf("LIBNJB problem: recieved %d extraneous bytes!\n", extraneous);
+		      printf("LIBNJB panic: recieved %d extraneous bytes!\n", extraneous);
 		    }
 		    chunk_size = remain;
 		  }
 		  remain -= chunk_size;
 		  if (remain < 0) {
-		    printf("Remain < 0!\n");
+		    printf("LIBNJB panic: Remain < 0!\n");
 		    remain = 0;
 		  }
 		}
@@ -946,29 +900,26 @@ int NJB_Get_Track (njb_t *njb, u_int32_t fileid, u_int32_t size,
 	  goto clean_up_and_return;
 	} 
 
-       	close(fd);
-	fd = 0;
-
-	if ( Move(temp, path) == -1 ) {
-       		NJB_ERROR2("Move", -1);
-       		NJB_ERROR(EO_CANTMOVE);
-       		ret = -1;
-       		goto clean_up_and_return;
-       	}
-
+	if ( path != NULL ) {
+	       	close(fd);
+		fd = 0;
+	}
 
 	ret = 0;
 
 clean_up_and_return:
-	if (block != NULL)
+
+	if ( block != NULL ) {
 	  free(block);
+	}
 
-	if ( fd ) 
+	if ( fd != 0 ) {
 	  close(fd);
+	}
 
-	if ( temp ) {
-	  unlink(temp);
-	  free(temp);
+	/* On error, remove the partial file. */
+	if ( ret == -1 ) {
+	  unlink(path);
 	}
 
 	__leave;
@@ -988,7 +939,6 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
   __dsub= "send_file";
   u_int64_t remain, offset;
   u_int32_t bp;
-  u_int32_t callbackcnt, callbackval;
   size_t bread;
   unsigned char *block;
   int fd;
@@ -998,8 +948,12 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
   struct stat sb;
   
   __enter;
-  
+
+#ifdef __WIN32__
+  if ( (fd= open(path, O_RDONLY|O_BINARY)) == -1 ) {
+#else
   if ( (fd= open(path, O_RDONLY)) == -1 ) {
+#endif
     NJB_RERROR("open", -1);
     NJB_ERROR(EO_SRCFILE);
     __leave;
@@ -1013,7 +967,7 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
     return -1;
   }
 
-  if ( (fp= fdopen(fd, "r")) == NULL ) {
+  if ( (fp= fdopen(fd, "rb")) == NULL ) {
     close(fd);
     NJB_RERROR("fdopen", -1);
     NJB_ERROR(EO_SRCFILE);
@@ -1022,7 +976,7 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
   }
   
   /* This space will be used as a reading ring buffer for the transfers */
-  block= (unsigned char *) malloc(NJB_BUFSIZ);
+  block = (unsigned char *) malloc(NJB_BUFSIZ);
   
   /* Terminate if memory block could not be allocated */
   if ( block == NULL ) {
@@ -1039,11 +993,6 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
   /* Set pointer at end of block, so that a new block will be read. */
   bp = NJB_BUFSIZ;
 
-  /* These two are used to make calls to the callback routine
-   * no more than CALLBACK_NUM times / song */
-  callbackcnt = size / CALLBACK_NUM;
-  callbackval = 0;
-  
   /* Keep sending while there is stuff to send and noone
    * interrupts us */
   while (remain && !abortxfer) {
@@ -1055,7 +1004,7 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
     if (PROTOCOL3_DEVICE(njb->device_type)) {
       maxblock = NJB3_SEND_FILE_BLOCK_SIZE;
     } else if (njb->device_type == NJB_DEVICE_NJB1) {
-      maxblock = MAX_XFER_BLOCK_SIZE;
+      maxblock = NJB_XFER_BLOCK_SIZE;
     }
     
     /* Next we will try to send this much */
@@ -1125,15 +1074,10 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
       return -1;
     }
 
-    /*
-      printf("Wrote %04x bytes to the USB bus\n", bwritten);
-    */
-    
     /* We use bwritten to cope with short transfers */
     remain -= (u_int64_t) bwritten;
     offset += (u_int64_t) bwritten;
     bp += bwritten;
-    callbackval += bwritten;
     
     if (PROTOCOL3_DEVICE(njb->device_type)) {
       /* DO NOTHING */
@@ -1147,10 +1091,9 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
       }
     }
 
-    if ( (callback != NULL) && (callbackval > callbackcnt) ) {
-      callbackval = 0;
+    if (callback != NULL) {
       if ( callback(offset, size, NULL, 0, data) == -1 )
-	abortxfer= 1;
+	abortxfer = 1;
     }
   }
   
@@ -1165,6 +1108,16 @@ static int send_file (njb_t *njb, const char *path, u_int64_t size, u_int32_t fi
       __leave;
       return -1;
     }
+    /* On abort, delete the partial file */
+    if ( abortxfer ) {
+      if ( njb3_delete_item(njb, fileid) == -1) {
+	__leave;
+	return -1;
+      }
+      NJB_ERROR(EO_ABORTED);
+      __leave;
+      return -1;
+    }
   } else {
     /* Complete transfer for NJB1  */
     while ( retry ) {
@@ -1495,10 +1448,11 @@ int NJB_Send_File (njb_t *njb, const char *path, const char *name,
 	if (njb->device_type == NJB_DEVICE_NJB1) {
 
 		if ( (phdr = datafile_pack(&df, &fhsize)) == NULL ) return -1;
-		fh.size= fhsize;
+		fh.size = fhsize;
 
 		if ( njb_send_datafile_tag(njb, &fh, phdr) == -1 ) {
-			if ( njb_error == EO_BADSTATUS ) NJB_ERROR(EO_XFERDENIED);
+			if ( njb_error == EO_BADSTATUS ) 
+			  NJB_ERROR(EO_XFERDENIED);
 			free(phdr);
 			__leave;
 			return -1;
@@ -1536,7 +1490,7 @@ int NJB_Send_File (njb_t *njb, const char *path, const char *name,
 	return 0;
 }
 
-
+/* This function is deprecated in favor of the new EAX API */
 eax_t *NJB_Get_EAX (njb_t *njb)
 {
 	__dsub= "NJB_Get_EAX";
@@ -1568,6 +1522,7 @@ eax_t *NJB_Get_EAX (njb_t *njb)
 	return eax;
 }
 
+/* This function is deprecated in favor of the new EAX API */
 int NJB_Refresh_EAX (njb_t *njb, eax_t *eax)
 {
 	__dsub= "NJB_Refresh_EAX";
@@ -1592,6 +1547,120 @@ int NJB_Refresh_EAX (njb_t *njb, eax_t *eax)
 	return 0;
 }
 
+/* The new EAX API */
+void NJB_Reset_Get_EAX_Type (njb_t *njb)
+{
+  __dsub= "NJB_Reset_Get_EAX_Type";
+  __enter;
+
+  njb_error_clear();
+
+  if (njb->device_type == NJB_DEVICE_NJB1) {
+    u_int32_t eaxsz;
+    if ( njb_get_eax_size(njb, &eaxsz) == -1 ) {
+      __leave;
+      return;
+    }
+    njb_read_eaxtypes(njb, eaxsz);
+  }
+  else if (PROTOCOL3_DEVICE(njb->device_type)) {
+    njb3_read_eaxtypes(njb);
+  }
+  __leave;
+  return;
+}
+
+njb_eax_t *NJB_Get_EAX_Type (njb_t *njb)
+{
+  __dsub= "NJB_Get_EAX_Type";
+  __enter;
+  if (njb->device_type == NJB_DEVICE_NJB1) {
+    njb_eax_t *eax = njb_get_nexteax(njb);
+    __leave;
+    return eax;
+  }
+  else if (PROTOCOL3_DEVICE(njb->device_type)) {
+    njb_eax_t *eax = njb3_get_nexteax(njb);
+    __leave;
+    return eax;
+  }
+  __leave;
+  return NULL;
+}
+
+void NJB_Destroy_EAX_Type (njb_eax_t *eax)
+
+{
+  __dsub= "NJB_Destroy_EAX_Type";
+  __enter;
+  destroy_eax_type(eax);
+  __leave;
+  return;
+}
+
+void NJB_Adjust_EAX (njb_t *njb, 
+		     u_int16_t eaxid, 
+		     u_int16_t patchindex,
+		     int16_t scalevalue)
+{
+  __dsub= "NJB_Adjust_EAX";
+  __enter;
+
+
+  if (njb->device_type == NJB_DEVICE_NJB1) {
+    int16_t sendvalue;
+    
+    // We assume that a scalevalue != 0 means that this is
+    // a scale to be modified.
+    if (scalevalue != 0x0000) {
+      sendvalue = scalevalue;
+    } else {
+      sendvalue = (int16_t) patchindex;
+    }
+    // Ignore return value
+    njb_adjust_sound(njb, (u_int8_t) eaxid, sendvalue);
+  }
+  else if (PROTOCOL3_DEVICE(njb->device_type)) {
+    u_int16_t sendindex;
+    u_int16_t active;
+    njb3_state_t *state = (njb3_state_t *) njb->protocol_state;
+
+    /*
+     * Volume is always active. 
+     * The rest need to add additional control of the EAX processor 
+     * so that it is activated/deactivated as needed.
+     */
+    if (eaxid == NJB3_VOLUME_FRAME_ID) {
+      active = 0x0001;
+    } else {
+      if (patchindex == 0x0000 && 
+	  scalevalue == 0x0000) {
+	if (state->eax_processor_active) {
+	  njb3_control_eax_processor(njb, 0x0000);
+	  state->eax_processor_active = 0x00;
+	}
+	active = 0x0000;
+      } else {
+	if (!state->eax_processor_active) {
+	  njb3_control_eax_processor(njb, 0x0001);
+	  state->eax_processor_active = 0x01;
+	}
+	active = 0x0001;
+      }
+    }
+
+    /* Patch 1 is actually patch 0, number 0 is a dummy for the "off" mode */
+    if (patchindex > 0x0000) {
+      sendindex = patchindex - 1;
+    } else {
+      sendindex = 0x0000;
+    }
+    njb3_adjust_eax(njb, eaxid, sendindex, active, scalevalue);
+  }
+  __leave;
+  return;
+}
+
 njb_time_t *NJB_Get_Time(njb_t *njb)
 {
 	__dsub= "NJB_Get_Time";
@@ -1650,6 +1719,12 @@ int NJB_Set_Time(njb_t *njb, njb_time_t *time)
 	return 0;
 }
 
+void NJB_Destroy_Time(njb_time_t *time)
+{
+  /* This is currently a very simple struct... */
+  free(time);
+}
+
 int NJB_Delete_Playlist (njb_t *njb, u_int32_t plid)
 {
 	__dsub= "NJB_Delete_Playlist";
@@ -2058,7 +2133,7 @@ int NJB_Seek_Track (njb_t *njb, u_int32_t position)
 	njb_error_clear();
 
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-	  ret= njb3_seek_track (njb, position);
+	  ret = njb3_seek_track (njb, position);
 	  __leave;
 	  return ret;
 	}
@@ -2160,15 +2235,18 @@ int NJB_Replace_Track_Tag (njb_t *njb, u_int32_t trackid, const char *codec,
 	 * to the same value seems to cause trouble.
 	 */
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
-		if(njb->extrarun_on_rename) njb->extrarun_on_rename = 0;
-		else {
+	        njb3_state_t *state = (njb3_state_t *) njb->protocol_state;
+
+		if(state->extrarun_on_rename) {
+		  state->extrarun_on_rename = 0;
+		} else {
 			char *tmptitle = NULL;
 			char *tmpalbum = NULL;
 			char *tmpgenre = NULL;
 			char *tmpartist = NULL;
 			char *tmpfname = NULL;
 
-			njb->extrarun_on_rename = 1;
+			state->extrarun_on_rename = 1;
 
 			if(title) {
 				tmptitle = malloc(strlen(title)+6);
@@ -2508,16 +2586,16 @@ int NJB_Adjust_Sound(njb_t *njb, u_int8_t effect, u_int16_t value)
 	if (PROTOCOL3_DEVICE(njb->device_type)) {
 
 	  if (effect == NJB_SOUND_SET_VOLUME) {
-	    ret = njb3_adjust_volume(njb, value);
+	    njb3_adjust_eax(njb, NJB3_VOLUME_FRAME_ID, 0x0000, 0x0001, value);
 	    __leave;
-	    return ret;
+	    return 0;
 	  }
 
 	  __leave;
 	  return 0;
 	}
 
-	ret= njb_adjust_sound(njb, effect, value);
+	ret = njb_adjust_sound(njb, effect, value);
 
 	__leave;
 	return ret;
@@ -2599,7 +2677,7 @@ int NJB_Set_Bitmap(njb_t *njb, const char * path) {
     FILE *f;
     char data[1088];
 
-    if ( (f= fopen(path, "r")) == NULL) {
+    if ( (f= fopen(path, "rb")) == NULL) {
       NJB_RERROR("fopen", -1);
       NJB_ERROR(EO_SRCFILE);
       __leave;
@@ -2676,3 +2754,12 @@ njbid_t *NJB_Ping(njb_t *njb)
 	__leave;
 	return njbid;
 }
+
+/* Retrieve the scanned keys */
+njb_keyval_t *NJB_Get_Keys(njb_t *njb)
+{
+  if (PROTOCOL3_DEVICE(njb->device_type)) {
+      return njb3_get_keys(njb);
+  }
+  return NULL;
+}
