@@ -1,9 +1,7 @@
-#include "../config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/ioctl.h>
 #include <string.h>
 #include <errno.h>
 #include "libnjb.h"
@@ -15,9 +13,41 @@
 #include "ioutil.h"
 #include "defs.h"
 #include "base.h"
+#include "eax.h"
 
 extern int __sub_depth;
 
+
+/**
+ * Initializes the basic state of the njb->protocol_state for the
+ * NJB1-device.
+ */
+int njb_init_state (njb_t *njb) {
+  __dsub= "njb_init_state";
+
+  njb_state_t *state;
+
+  __enter;
+
+  state = malloc(sizeof(njb_state_t));
+  if (state == NULL) {
+    __leave;
+    return -1;
+  }
+
+  state->session_updated = 0;
+  state->libcount = 0;
+  state->first_eax = NULL;
+  state->next_eax = NULL;
+  state->reset_get_track_tag = 0;
+  state->reset_get_playlist = 0;
+  state->reset_get_datafile_tag = 0;
+  njb->protocol_state = (unsigned char *) state;
+
+  __leave;
+  return 0;
+}
+
 /*
  * Beginning with firmware 2.93, very strange things started happening.
  * For certian functions, a non-zero status code does not automatically
@@ -37,7 +67,7 @@ int njb_set_library_counter (njb_t *njb, u_int64_t count)
 	memset(data, 0, 8);
 	from_64bit_to_njb1_bytes(count, &data[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SET_LIBRARY_COUNTER, 0, 0, 8, data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -62,7 +92,7 @@ int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 	memset(lcount, 0, sizeof(njblibctr_t));
 	memset(data, 0, 25);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_LIBRARY_COUNTER, 0, 0, 25, data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -99,7 +129,7 @@ int njb_ping (njb_t *njb, njbid_t *njbid)
 	
 	memset(data, 0, 58);
 
-       	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_PING, 0, 0, 0, NULL) == -1 ) {
        		NJB_ERROR(EO_USBCTL);
        		__leave;
@@ -150,7 +180,7 @@ int njb_verify_last_command (njb_t *njb)
 
 	__enter;
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_VERIFY_LAST_CMD, 0, 0, 1, &data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -178,7 +208,7 @@ int njb_capture (njb_t *njb, int which)
 
 	__enter;
 
-       	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        		mode, 0, 0, 1, &data) == -1 ) {
 
        		NJB_ERROR(EO_USBCTL);
@@ -205,7 +235,7 @@ int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 
 	memset(data, 0, 9);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -252,7 +282,7 @@ songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 	lsw= get_lsw(tagh->trackid);
 	msw= get_msw(tagh->trackid);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_TRACK_TAG, msw, lsw, 0, NULL) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		free(data);
@@ -287,7 +317,7 @@ int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 
 	memset(data, 0, 9);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -338,7 +368,7 @@ playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 	lsw= get_lsw(plh->plid);
 	msw= get_msw(plh->plid);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_PLAYLIST, msw, lsw, 0, NULL) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -378,7 +408,7 @@ int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free)
 
 	memset(data, 0, 17);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_DISK_USAGE, 0, 0, 17, data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -412,7 +442,7 @@ int njb_get_owner_string (njb_t *njb, owner_string name)
 	__enter;
 
 	memset(data,0,OWNER_STRING_LENGTH+1);
-       	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_GET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
        		NJB_ERROR(EO_USBCTL);
@@ -451,7 +481,7 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
 
 	__enter;
 
-       	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
        		NJB_ERROR(EO_USBCTL);
@@ -485,7 +515,7 @@ int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 
 	memset(data, 0, 9);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -532,7 +562,7 @@ datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 	lsw= get_lsw(dfh->dfid);
 	msw= get_msw(dfh->dfid);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_DATAFILE_TAG, msw, lsw, 0, NULL) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -563,11 +593,11 @@ datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 	return df;
 }
 
-/*
+/**
  * offset    is the offset into the file, starting at 0.
  * bsize     indicates the recieved buffer max size.
  * bp        points to the recieve buffer
- *           (atleast MAX_XFER_BLOCK_SIZE + XFER_BLOCK_HEADER_SIZE)
+ *           (atleast NJB_XFER_BLOCK_SIZE + NJB_XFER_BLOCK_HEADER_SIZE)
  * lastsort  indicates if last transfer was short (ended before
  *           requested number of bytes were recieved).
  *
@@ -587,7 +617,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	__enter;
 
 	/* Too large transfer requested */
-	if ( bsize > MAX_XFER_BLOCK_SIZE ) {
+	if ( bsize > NJB_XFER_BLOCK_SIZE ) {
 		NJB_ERROR(EO_TOOBIG);
 		__leave;
 		return -1;
@@ -601,7 +631,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	from_32bit_to_njb1_bytes(offset, &filedata[0]);
 	from_32bit_to_njb1_bytes(bsize, &filedata[4]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		       NJB_CMD_RECEIVE_FILE_BLOCK, 0, 0, 8, filedata) == -1 ) {
 	  
 	  NJB_ERROR(EO_USBCTL);
@@ -610,11 +640,11 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	}
 
 	/* Read the status and block header first */
-	if ( (bread = usb_pipe_read(njb, bp, bsize + XFER_BLOCK_HEADER_SIZE)) == -1 ) {
+	if ( (bread = usb_pipe_read(njb, bp, bsize + NJB_XFER_BLOCK_HEADER_SIZE)) == -1 ) {
 	  NJB_ERROR(EO_USBBLK);
 	  __leave;
 	  return -1;
-	} else if ( bread < bsize + XFER_BLOCK_HEADER_SIZE ) {
+	} else if ( bread < bsize + NJB_XFER_BLOCK_HEADER_SIZE ) {
 	  /* This was a short transfer, OK... */
 	  /* printf("Short transfer on pipe.\n"); */
 	}
@@ -641,7 +671,7 @@ int njb_request_file (njb_t *njb, u_int32_t fileid)
 	memset(data, 0, 4);
 	from_32bit_to_njb1_bytes(fileid, &data[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REQUEST_TRACK, 1, 0, 4, data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -659,7 +689,7 @@ int njb_transfer_complete (njb_t *njb)
 
 	__enter;
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_TRANSFER_COMPLETE, 0, 0, 1, &data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -691,7 +721,7 @@ int njb_send_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 	
 	from_32bit_to_njb1_bytes(tagh->size, &utagsz[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_TRACK_TAG, 0, 0, 4, utagsz) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -746,9 +776,10 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 	memset(utagsz, 0, 4);
 	memset(data, 0, 5);
 
-	from_32bit_to_njb1_bytes(dfh->size, &utagsz[0]);
+	/* Add five here for the header overhead */
+	from_32bit_to_njb1_bytes(dfh->size+5, &utagsz[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_DATAFILE_TAG, 0, 0, 4, utagsz) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -788,7 +819,7 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 
 	/* Read back datafile ID */
 
-	bread= usb_pipe_read(njb, data, 5);
+	bread = usb_pipe_read(njb, data, 5);
 	if ( bread == -1 ) {
 		NJB_ERROR(EO_USBBLK);
 		__leave;
@@ -812,7 +843,7 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 }
 
 /*
- * This function transfers a block of <= MAX_XFER_BLOCK_SIZE to the
+ * This function transfers a block of <= NJB_XFER_BLOCK_SIZE to the
  * jukebox and returns the number of bytes actually sent. Short transfers
  * are OK but must be accounted for.
  */
@@ -829,7 +860,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 
 	__enter;
 
-	if ( blocksize > MAX_XFER_BLOCK_SIZE ) {
+	if ( blocksize > NJB_XFER_BLOCK_SIZE ) {
 		NJB_ERROR(EO_TOOBIG);
 		__leave;
 		return -1;
@@ -844,7 +875,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	  /* Only if something fails, retry */
 	  retry = 0;
 
-	  if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	  if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 			 NJB_CMD_SEND_FILE_BLOCK, lsw, msw, 1, &status)
 	       == -1 ) {
 	    
@@ -896,7 +927,7 @@ int njb_stop_play (njb_t *njb)
 
 	__enter;
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_STOP_PLAY, 0, 0, 1, &data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -923,7 +954,7 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 
 	memset(data, 0, 5);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_EAX_SIZE, 0, 0, 5, data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -943,6 +974,7 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 	return 0;
 }
 
+/* This is an old, deprecated routine */
 eax_t *njb_get_eax (njb_t *njb, u_int32_t size)
 {
 	__dsub= "njb_get_eax";
@@ -965,7 +997,7 @@ eax_t *njb_get_eax (njb_t *njb, u_int32_t size)
 	
 	from_32bit_to_njb1_bytes(size, &usize[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
 		0, 0, 4, usize) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1006,6 +1038,83 @@ eax_t *njb_get_eax (njb_t *njb, u_int32_t size)
 	return eax;
 }
 
+/* This is the new EAX API */
+void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
+{
+  __dsub= "njb_read_eaxtypes";
+
+  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+  unsigned char *data;
+  ssize_t bread;
+  u_int32_t actsize;
+  unsigned char usize[4];
+
+  __enter;
+
+  data = (unsigned char *) malloc(size);
+  if ( data == NULL ) {
+    NJB_ERROR(EO_NOMEM);
+    __leave;
+    return;
+  }
+  memset(data, 0, size);
+  memset(usize, 0, 4);
+	
+  from_32bit_to_njb1_bytes(size, &usize[0]);
+
+  if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
+		 0, 0, 4, usize) == -1 ) {
+    
+    NJB_ERROR(EO_USBCTL);
+    free(data);
+    __leave;
+    return;
+  }
+
+  bread = usb_pipe_read(njb, data, size+5);
+  if ( bread == -1 ) {
+    NJB_ERROR(EO_USBBLK);
+    free(data);
+    __leave;
+    return;
+  } else if ( bread < size ) {
+    NJB_ERROR(EO_RDSHORT);
+    free(data);
+    __leave;
+    return;
+  }
+
+  if ( data[0] ) {
+    NJB_STATUS((int) data[0]);
+    free(data);
+    __leave;
+    return;
+  }
+
+  actsize = njb1_bytes_to_32bit(&data[1]);
+
+  /* Unpack the EAX structure */
+  eax_unpack_new_api(&data[5], actsize, state);
+
+  free(data);
+
+  __leave;
+  return;
+}
+
+njb_eax_t *njb_get_nexteax(njb_t *njb)
+{
+  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+  njb_eax_t *ret;
+
+  ret = state->next_eax;
+  if (ret != NULL) {
+    state->next_eax = ret->next;
+  }
+  return ret;
+}
+
+/* This routine is dangerous. Do not call it... */
 int njb_refresh_eax(njb_t *njb, eax_t *eax)
 {
 	__dsub= "njb_refresh_eax";
@@ -1023,7 +1132,7 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 	}
 	memset(data, 0, 4);
 	
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 			NJB_CMD_GET_EAX, 1, 0, 4, data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		free(data);
@@ -1091,7 +1200,7 @@ njb_time_t *njb_get_time(njb_t *njb)
 	}
 	memset(data, 0, 17);
 	
-       	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        			NJB_CMD_GET_TIME, 0, 0, 17, data) == -1 ) {
        		NJB_ERROR(EO_USBCTL);
 		free(data);
@@ -1124,7 +1233,7 @@ int njb_set_time(njb_t *njb, njb_time_t *time)
        		return -1;
        	}
 
-       	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+       	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_TIME, 0, 0, 16, data) == -1 ) {
 
        		NJB_ERROR(EO_USBCTL);
@@ -1159,7 +1268,7 @@ int njb_create_playlist (njb_t *njb, char *name, u_int32_t *plid)
 	msw= get_msw( (u_int32_t) size );
 	lsw= get_lsw( (u_int32_t) size );
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_CREATE_PLAYLIST, lsw, msw, 0, NULL) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -1211,7 +1320,7 @@ int njb_delete_playlist (njb_t *njb, u_int32_t plid)
 	msw= get_msw(plid);
 	lsw= get_lsw(plid);
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_PLAYLIST, msw, lsw, 1, &data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1250,7 +1359,7 @@ int njb_rename_playlist (njb_t *njb, u_int32_t plid, char *name)
 	from_32bit_to_njb1_bytes(plid, &data[0]);
 	from_32bit_to_njb1_bytes(size, &data[4]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_RENAME_PLAYLIST, 0, 0, 8, data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1284,7 +1393,7 @@ int njb_add_track_to_playlist (njb_t *njb, u_int32_t plid, u_int32_t trid)
 	from_32bit_to_njb1_bytes(trid, &data[2]);
 	from_32bit_to_njb1_bytes(plid, &data[6]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADD_TRACK_TO_PLAYLIST, 0, 0, 10, data) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1329,7 +1438,7 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 	from_32bit_to_njb1_bytes(plid, &data[0]);
 	from_16bit_to_njb1_bytes(ntracks, &data[4]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADD_MULTIPLE_TRACKS_TO_PLAYLIST, 0, 0, 6, data) 
 		== -1 ) {
 
@@ -1369,7 +1478,7 @@ int njb_delete_track (njb_t *njb, u_int32_t trackid)
 	msw= get_msw(trackid);
 	lsw= get_lsw(trackid);
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_TRACK, msw, lsw, 1, &data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -1397,7 +1506,7 @@ int njb_delete_datafile (njb_t *njb, u_int32_t fileid)
 	msw= get_msw(fileid);
 	lsw= get_lsw(fileid);
 
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_DATAFILE, msw, lsw, 1, &data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -1425,7 +1534,7 @@ int njb_play_or_queue (njb_t *njb, u_int32_t trackid, int cmd)
 	
 	from_32bit_to_njb1_bytes(trackid, &data[0]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER, cmd, 0, 0, 4, data)
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, cmd, 0, 0, 4, data)
 		 == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1446,7 +1555,7 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 
 	memset(data, 0, 3);
 	
-	if ( usb_setup(&njb->ctl, UT_READ_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_ELAPSED_TIME, 0, 0, 3, data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
@@ -1492,7 +1601,7 @@ int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 	from_32bit_to_njb1_bytes(tagh->trackid, &data[0]);
 	memcpy(&data[4], tag, tagh->size);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REPLACE_TRACK_TAG, lsw, msw, 0, NULL) == -1 ) {
 
 		NJB_ERROR(EO_USBCTL);
@@ -1519,19 +1628,22 @@ int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 	return 0;
 }
 
-int njb_adjust_sound(njb_t *njb, u_int8_t effect, u_int16_t value)
+int njb_adjust_sound(njb_t *njb, u_int8_t effect, int16_t value)
 {
 	__dsub= "njb_adjust_sound";
 	unsigned char data[3];
+	u_int16_t sendvalue = ((u_int16_t) value) & 0xff;
 
 	__enter;
 
 	memset(data, 0, 3);
-
 	data[0] = effect;
-	from_16bit_to_njb1_bytes(value, &data[1]);
 
-	if ( usb_setup(&njb->ctl, UT_WRITE_VENDOR_OTHER,
+	printf("Effect %d, sending value %d (%04X)\n", effect, sendvalue, sendvalue);
+
+	from_16bit_to_njb1_bytes(sendvalue, &data[1]);
+
+	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADJUST_SOUND, 0, 0, 3, data) == -1 ) {
 		NJB_ERROR(EO_USBCTL);
 		__leave;
