@@ -1,4 +1,3 @@
-#include "../config.h"
 #include <stdlib.h>
 #include <string.h>
 #include "libnjb.h"
@@ -8,6 +7,19 @@
 #include "byteorder.h"
 #include "unicode.h"
 
+/*
+ * These unicode tags appeared in a users NJB1 jukebox,
+ * we still don't know what program that writes them, and
+ * we do not support writing them, but if they're present,
+ * they are recognized and used.
+ */
+#define FR_UNI_TITLE	"UNI_TITLE"
+#define FR_UNI_ALBUM	"UNI_ALBUM"
+#define FR_UNI_GENRE	"UNI_GENRE"
+#define FR_UNI_ARTIST	"UNI_ARTIST"
+/* Haven't seen this one being used, but add it anyway. */
+#define FR_UNI_FNAME	"UNI_FNAME"
+
 extern int njb_error;
 extern int __sub_depth;
 extern int njb_unicode_flag;
@@ -36,49 +48,135 @@ songid_t *songid_unpack (void *data, size_t nbytes)
 	size_t index;
 	songid_t *song;
 	u_int16_t i, nframes;
+	/* The unicode tags shall take precedence */
+	int had_uni_title = 0;
+	int had_uni_album = 0;
+	int had_uni_genre = 0;
+	int had_uni_artist = 0;
+	int had_uni_fname = 0;
 
-	song= songid_new();
+	song = songid_new();
 	if ( song == NULL ) return NULL;
 
-	nframes = (dp[1] << 8) + dp[0];
+	nframes = njb1_bytes_to_16bit(&dp[0]);
 
 	index = 2;
-	for (i= 0; i<nframes; i++) {
+
+	/* Loop through the frames of a single song ID */
+	for (i = 0; i < nframes; i++) {
+	        /* Each frame has a label, a value and a type */
 		u_int16_t lsize, vsize, type;
+		char *label = NULL;
+		char *value = NULL;
 		songid_frame_t *frame = NULL;
 
 		type = njb1_bytes_to_16bit(&dp[index]);
 		lsize = njb1_bytes_to_16bit(&dp[index+2]);
 		vsize = njb1_bytes_to_16bit(&dp[index+4]);
 		index += 8;
+		label = &dp[index];
+		value = &dp[index+lsize];
 
 		/* Handle Unicode conversion on ASCII fields, or default
 		 * to ISO 8859-1 */
-		if (type == ID_DATA_ASCII && njb_unicode_flag == NJB_UC_UTF8) {
+		if (type == ID_DATA_ASCII) {
+		  if (njb_unicode_flag == NJB_UC_UTF8) {
+		    if ( (!strcmp(label, FR_TITLE) && had_uni_title) ||
+			 (!strcmp(label, FR_ALBUM) && had_uni_album) ||
+			 (!strcmp(label, FR_GENRE) && had_uni_genre) ||
+			 (!strcmp(label, FR_ARTIST) && had_uni_artist) ||
+			 (!strcmp(label, FR_FNAME) && had_uni_fname) ) {
+		      frame = NULL;
+		    } else {
+		      char *utf8str = NULL;
+		      int utf8len;
+		      
+		      utf8str = strtoutf8(value);
+		      if (utf8str == NULL) {
+			songid_destroy(song);
+			njb_error_add(__sub, EO_NOMEM);
+			return NULL;
+		      }
+		      utf8len= strlen(utf8str)+1;
+		      frame = songid_frame_new(lsize, label, type, utf8len,
+					       utf8str);
+		      free(utf8str);
+		    }
+		  } else {
+		    /* No Unicode conversion, default by treating as ISO 8859-1 */
+		    frame = songid_frame_new(lsize, label, type, strlen(value)+1,
+					     value);
+		  }
+		/*
+		 * The Unicode labels appeared in later software, and shall take
+		 * precedence if present.
+		 */
+		} else if (type == ID_DATA_UNI) {
 		  char *utf8str = NULL;
 		  int utf8len;
 
-		  utf8str= strtoutf8(&dp[index+lsize]);
+		  utf8str = ucs2tostr(value);
 		  if (utf8str == NULL) {
 		    songid_destroy(song);
 		    njb_error_add(__sub, EO_NOMEM);
 		    return NULL;
 		  }
-		  utf8len= strlen(utf8str)+1;
-		  frame= songid_frame_new(lsize, &dp[index], type, utf8len,
-					  utf8str);
+		  utf8len = strlen(utf8str)+1;
+		  /*
+		   * After converting the Unicode frame into an UTF8 ASCII 
+		   * frame, we use the standard labels to simplify the API.
+		   */
+		  if (!strcmp(label, FR_UNI_TITLE)) {
+		    frame = songid_frame_new(strlen(FR_TITLE)+1, FR_TITLE, ID_DATA_ASCII, utf8len,
+					     utf8str);
+		    had_uni_title = 1;
+		    printf("Found Unicode title: %s\n", utf8str);
+		  } else if (!strcmp(label, FR_UNI_ALBUM)) {
+		    frame = songid_frame_new(strlen(FR_ALBUM)+1, FR_ALBUM, ID_DATA_ASCII, utf8len,
+					     utf8str);
+		    had_uni_album = 1;
+		  } else if (!strcmp(label, FR_UNI_GENRE)) {
+		    frame = songid_frame_new(strlen(FR_GENRE)+1, FR_GENRE, ID_DATA_ASCII, utf8len,
+					     utf8str);
+		    had_uni_genre = 1;
+		  } else if (!strcmp(label, FR_UNI_ARTIST)) {
+		    frame = songid_frame_new(strlen(FR_ARTIST)+1, FR_ARTIST, ID_DATA_ASCII, utf8len,
+					     utf8str);
+		    had_uni_artist = 1;
+		  } else if (!strcmp(label, FR_UNI_FNAME)) {
+		    frame = songid_frame_new(strlen(FR_FNAME)+1, FR_FNAME, ID_DATA_ASCII, utf8len,
+					     utf8str);
+		    had_uni_fname = 1;
+		  }
 		  free(utf8str);
-		} else {
-		  frame= songid_frame_new(lsize, &dp[index], type, vsize,
-					  &dp[index+lsize]);
+		} else { /* This means it is a numeric value */
+		  /*
+		   * FIXME: check that this can fix some endianness issues - 
+		   * the values arrive in device endianness 
+		   */
+		  if (vsize == 2) {
+		    u_int16_t *dummy = (u_int16_t *) value;
+		    *dummy = njb1_bytes_to_16bit(value);
+		  } else if (vsize == 4) {
+		    u_int32_t *dummy = (u_int32_t *) value;
+		    *dummy = njb1_bytes_to_32bit(value);
+		  } else if (vsize == 8) {
+		    u_int64_t *dummy = (u_int64_t *) value;
+		    *dummy = njb1_bytes_to_64bit(value);
+		  } else {
+		    /* Just copy it, whatever it is. */
+		  }
+		  frame = songid_frame_new(lsize, label, type, vsize, value);
 		}
-		if ( frame == NULL ) {
-			songid_destroy(song);
-			njb_error_add(__sub, EO_NOMEM);
-			return NULL;
+		/*
+		 * The frame will only be NULL if there was a Unicode frame
+		 * taking precedence over the ISO 8859-1 frame.
+		 */
+		if (frame != NULL) {
+		  songid_addframe(song, frame);
 		}
-		songid_addframe(song, frame);
-		index += (lsize+vsize);
+
+		index += (lsize + vsize);
 
 		if ( index > nbytes ) {
 			songid_destroy(song);
@@ -110,7 +208,8 @@ unsigned char *songid_pack (songid_t *song, u_int32_t *tagsize)
 	index = 2;
 
 	songid_reset_getframe(song);
-	while ((frame= songid_getframe(song))) {
+	/* FIXME: add UNI_ unicode frames for all applicable frames, except filename */
+	while ((frame = songid_getframe(song))) {
 	        /* Add type and size for this frame */
 		from_16bit_to_njb1_bytes(frame->type, &tagbuffer[index]);
 		from_16bit_to_njb1_bytes(frame->labelsz, &tagbuffer[index+2]);
@@ -162,7 +261,7 @@ unsigned char *songid_pack (songid_t *song, u_int32_t *tagsize)
 		    from_32bit_to_njb1_bytes(*(u_int32_t *)frame->data,
 					     &tagbuffer[index]);
 		  } else if ( frame->datasz == 8) {
-		    from_32bit_to_njb1_bytes(*(u_int64_t *)frame->data,
+		    from_64bit_to_njb1_bytes(*(u_int64_t *)frame->data,
 					     &tagbuffer[index]);
 		  } else {
 		    /* Only numerical frames of size 1,2,4 and 8 are allowed. */
@@ -195,10 +294,17 @@ static u_int32_t valtoint32(unsigned char *data, int valsz)
   u_int32_t retval = 0;
 
   if (valsz == 2) {
-    retval = (data[1]<<8) + data[0];
+    u_int16_t dummy;
+    u_int16_t *temp = (u_int16_t *) data;
+
+    /* retval = (data[1]<<8) + data[0]; - old bad endianness code */
+    dummy = *temp;
+    retval = (u_int32_t) dummy;
   }
   else if (valsz == 4) {
-    retval = (data[3]<<24) + (data[2]<<16) + (data[1]<<8) + data[0];
+    u_int32_t *temp = (u_int32_t *) data;
+    /* retval = (data[3]<<24) + (data[2]<<16) + (data[1]<<8) + data[0]; - old bad endianess code */
+    retval = *temp;
   }
   return retval;
 }
@@ -267,28 +373,28 @@ unsigned char *songid_pack3 (songid_t *song, u_int32_t *tagsize)
 	  else if (!strcmp(frame->label, FR_LENGTH)) {
 	    /* 6 bytes, 2 length 2 tagid 2bytes length */
 	    track_length = valtoint32(frame->data, frame->datasz);
-	    *tagsize+= 6;
+	    *tagsize += 6;
 	  }
 	  /* For strings, stringlengh * 2 + 4 bytes */
 	  else if (!strcmp(frame->label, FR_TITLE)) {
 	    track_title = strtoucs2(frame->data);
-	    *tagsize+= (6 + 2*ucs2strlen(track_title));
+	    *tagsize += (6 + 2*ucs2strlen(track_title));
 	  }
 	  else if (!strcmp(frame->label, FR_ALBUM)) {
 	    track_album = strtoucs2(frame->data);
-	    *tagsize+= (6 + 2*ucs2strlen(track_album));
+	    *tagsize += (6 + 2*ucs2strlen(track_album));
 	  }
 	  else if (!strcmp(frame->label, FR_ARTIST)) {
 	    track_artist = strtoucs2(frame->data);
-	    *tagsize+= (6 + 2*ucs2strlen(track_artist));
+	    *tagsize += (6 + 2*ucs2strlen(track_artist));
 	  }
 	  else if (!strcmp(frame->label, FR_GENRE)) {
 	    track_genre = strtoucs2(frame->data);
-	    *tagsize+= (6 + 2*ucs2strlen(track_genre));
+	    *tagsize += (6 + 2*ucs2strlen(track_genre));
 	  }
 	  else if (!strcmp(frame->label, FR_FNAME)) {
 	    track_fname = strtoucs2(frame->data);
-	    *tagsize+= (6 + 2*ucs2strlen(track_fname));
+	    *tagsize += (6 + 2*ucs2strlen(track_fname));
 	  }
 	  else if (!strcmp(frame->label, FR_CODEC)) {
 	    /* 6 bytes, 2 length 2 tagid 2bytes year */
@@ -519,36 +625,62 @@ void songid_frame_dump (songid_frame_t *frame, FILE *fp)
 		fprintf(fp, "%s\n", data);
 		free(data);
 	} else {
-		if ( frame->datasz > 4 ) {
-			fprintf(fp, "(can't display 64-bit int)\n");
-		} else {
-			fprintf(fp, "%u\n", songid_frame_data32(frame));
-		}
+	  /* 64 bit numbers... */
+	  if (frame->datasz == 8) {
+	    u_int64_t *valuep = (u_int64_t *) frame->data;
+	    u_int64_t value = *valuep;
+#ifdef __WIN32__
+	    fprintf(fp, "%I64u\n", value);
+#else
+	    fprintf(fp, "%llu\n", value);
+#endif
+	  } else if (frame->datasz == 4) {
+	    u_int32_t *valuep = (u_int32_t *) frame->data;
+	    u_int32_t value = *valuep;
+	    fprintf(fp, "%u\n", value);
+	  } else if (frame->datasz == 2) {
+	    u_int16_t *valuep = (u_int16_t *) frame->data;
+	    u_int16_t value = *valuep;
+	    fprintf(fp, "%u\n", value);
+	  } else if (frame->datasz == 1) {
+	    u_int8_t *valuep = (u_int8_t *) frame->data;
+	    u_int8_t value = *valuep;
+	    fprintf(fp, "%u\n", value);
+	  } else {
+	    fprintf(fp, "(weird data word size: %d bytes, cannot display!)\n", frame->datasz);
+	  }
 	}
 }
 
-u_int32_t songid_frame_data32 (songid_frame_t *frame)
-{
-	if ( frame->datasz == 1 ) {
-		char tval= ((char *) frame->data)[0];
-		return (u_int32_t) tval;
-	} else if ( frame->datasz == 2 ) {
-		u_int16_t tval;
-		memcpy(&tval, frame->data, 2);
-		return (u_int32_t) tval;
-	} else if ( frame->datasz == 4 ) {
-		u_int32_t tval;
-		memcpy(&tval, frame->data, 4);
-		return tval;
-	} 
-
-	return 0;
-}
-
 /*
  * Info methods
  */
 
+u_int32_t songid_frame_data32(songid_frame_t *frame)
+{
+  if (frame->datasz == 8) {
+    u_int64_t *valuep = (u_int64_t *) frame->data;
+    u_int64_t value = *valuep;
+    /* This makes us loose all high digits! */
+    return (u_int32_t) value;
+  } else if (frame->datasz == 4) {
+    u_int32_t *valuep = (u_int32_t *) frame->data;
+    u_int32_t value = *valuep;
+    return value;
+  } else if (frame->datasz == 2) {
+    u_int16_t *valuep = (u_int16_t *) frame->data;
+    u_int16_t value = *valuep;
+    return (u_int32_t) value;
+  } else if (frame->datasz == 1) {
+    u_int8_t *valuep = (u_int8_t *) frame->data;
+    u_int8_t value = *valuep;
+    return (u_int32_t) value;
+  } else {
+    printf("LIBNJB panic: weird data word size: %d bytes, cannot convert!)\n", frame->datasz);
+  }
+  return 0;
+}
+
 u_int32_t songid_size (songid_t *song)
 {
 	songid_frame_t *frame;
